@@ -1,6 +1,4 @@
-from enum import Enum
 import os
-import sys
 
 from shapely.geometry import (
     LineString,
@@ -13,7 +11,6 @@ from shapely.geometry import (
 
 import geopandas
 from geopandas import GeoDataFrame
-from geopandas.io.file import _FIONA18
 
 from geopandas.testing import assert_geodataframe_equal
 import pytest
@@ -68,11 +65,6 @@ point_3D = Point(-73.553785, 45.508722, 300)
 # TEST TOOLING
 
 
-class _Fiona(Enum):
-    below_1_8 = "fiona_below_1_8"
-    above_1_8 = "fiona_above_1_8"
-
-
 class _ExpectedError:
     def __init__(self, error_type, error_message_match):
         self.type = error_type
@@ -89,19 +81,16 @@ class _ExpectedErrorBuilder:
         )
 
 
-def _expect_writing(gdf, ogr_driver, fiona_version):
-    return _ExpectedErrorBuilder(_composite_key(gdf, ogr_driver, fiona_version))
+def _expect_writing(gdf, ogr_driver):
+    return _ExpectedErrorBuilder(_composite_key(gdf, ogr_driver))
 
 
-def _composite_key(gdf, ogr_driver, fiona_version):
-    return frozenset([id(gdf), ogr_driver, fiona_version.value])
+def _composite_key(gdf, ogr_driver):
+    return frozenset([id(gdf), ogr_driver])
 
 
-def _expected_error_on(gdf, ogr_driver, is_fiona_above_1_8):
-    if is_fiona_above_1_8:
-        composite_key = _composite_key(gdf, ogr_driver, _Fiona.above_1_8)
-    else:
-        composite_key = _composite_key(gdf, ogr_driver, _Fiona.below_1_8)
+def _expected_error_on(gdf, ogr_driver):
+    composite_key = _composite_key(gdf, ogr_driver)
     return _expected_exceptions.get(composite_key, None)
 
 
@@ -141,15 +130,7 @@ _geodataframes_to_write.append(gdf)
 # 'ESRI Shapefile' driver supports writing LineString/MultiLinestring and
 # Polygon/MultiPolygon but does not mention Point/MultiPoint
 # see https://www.gdal.org/drv_shapefile.html
-for driver in ("ESRI Shapefile", "GPKG"):
-    _expect_writing(gdf, driver, _Fiona.below_1_8).to_raise(
-        ValueError,
-        "Record's geometry type does not match collection schema's geometry "
-        "type: 'MultiPoint' != 'Point'",
-    )
-_expect_writing(gdf, "ESRI Shapefile", _Fiona.above_1_8).to_raise(
-    RuntimeError, "Failed to write record"
-)
+_expect_writing(gdf, "ESRI Shapefile").to_raise(RuntimeError, "Failed to write record")
 
 # ------------------
 # gdf with LineStrings
@@ -173,11 +154,6 @@ gdf = GeoDataFrame(
     geometry=[MultiLineString(city_hall_walls), city_hall_walls[0]],
 )
 _geodataframes_to_write.append(gdf)
-_expect_writing(gdf, "GPKG", _Fiona.below_1_8).to_raise(
-    ValueError,
-    "Record's geometry type does not match collection schema's geometry "
-    "type: 'MultiLineString' != 'LineString'",
-)
 
 # ------------------
 # gdf with Polygons
@@ -206,11 +182,6 @@ gdf = GeoDataFrame(
     ],
 )
 _geodataframes_to_write.append(gdf)
-_expect_writing(gdf, "GPKG", _Fiona.below_1_8).to_raise(
-    ValueError,
-    "Record's geometry type does not match collection schema's geometry "
-    "type: 'MultiPolygon' != 'Polygon'",
-)
 
 # ------------------
 # gdf with null geometry and Point
@@ -243,13 +214,7 @@ gdf = GeoDataFrame(
 )
 _geodataframes_to_write.append(gdf)
 # Not supported by 'ESRI Shapefile' driver
-for driver in ("ESRI Shapefile", "GPKG"):
-    _expect_writing(gdf, driver, _Fiona.below_1_8).to_raise(
-        AttributeError, "'list' object has no attribute 'lstrip'"
-    )
-_expect_writing(gdf, "ESRI Shapefile", _Fiona.above_1_8).to_raise(
-    RuntimeError, "Failed to write record"
-)
+_expect_writing(gdf, "ESRI Shapefile").to_raise(RuntimeError, "Failed to write record")
 
 # ------------------
 # gdf with all 2D shape types and 3D Point mixed together
@@ -268,13 +233,7 @@ gdf = GeoDataFrame(
 )
 _geodataframes_to_write.append(gdf)
 # Not supported by 'ESRI Shapefile' driver
-for driver in ("ESRI Shapefile", "GPKG"):
-    _expect_writing(gdf, driver, _Fiona.below_1_8).to_raise(
-        AttributeError, "'list' object has no attribute 'lstrip'"
-    )
-_expect_writing(gdf, "ESRI Shapefile", _Fiona.above_1_8).to_raise(
-    RuntimeError, "Failed to write record"
-)
+_expect_writing(gdf, "ESRI Shapefile").to_raise(RuntimeError, "Failed to write record")
 
 
 @pytest.fixture(params=_geodataframes_to_write)
@@ -290,20 +249,13 @@ def ogr_driver(request):
 def test_to_file_roundtrip(tmpdir, geodataframe, ogr_driver):
     output_file = os.path.join(str(tmpdir), "output_file")
 
-    expected_error = _expected_error_on(geodataframe, ogr_driver, _FIONA18)
+    expected_error = _expected_error_on(geodataframe, ogr_driver)
     if expected_error:
-        with pytest.raises(expected_error.type, match=expected_error.match):
+        with pytest.raises(RuntimeError, match="Failed to write record"):
             geodataframe.to_file(output_file, driver=ogr_driver)
     else:
         geodataframe.to_file(output_file, driver=ogr_driver)
 
         reloaded = geopandas.read_file(output_file)
 
-        check_column_type = "equiv"
-        if sys.version_info[0] < 3:
-            # do not check column types in python 2 (mixed string/unicode)
-            check_column_type = False
-
-        assert_geodataframe_equal(
-            geodataframe, reloaded, check_column_type=check_column_type
-        )
+        assert_geodataframe_equal(geodataframe, reloaded, check_column_type="equiv")
