@@ -1,7 +1,7 @@
 from .crs import explicit_crs_from_epsg
 from .geocoding import geocode, reverse_geocode
 from .overlay import overlay
-from .sjoin import sjoin
+from .sjoin import sjoin, sjoin_nearest
 from .util import collect
 from .clip import clip
 
@@ -12,5 +12,6 @@ __all__ = [
     "overlay",
     "reverse_geocode",
     "sjoin",
+    "sjoin_nearest",
     "clip",
 ]
