@@ -1,13 +1,15 @@
 import pandas as pd
-from shapely.geometry import MultiPoint, MultiLineString, MultiPolygon
+
+from shapely.geometry import MultiLineString, MultiPoint, MultiPolygon
 from shapely.geometry.base import BaseGeometry
 
 _multi_type_map = {
-    'Point': MultiPoint,
-    'LineString': MultiLineString,
-    'Polygon': MultiPolygon
+    "Point": MultiPoint,
+    "LineString": MultiLineString,
+    "Polygon": MultiPolygon,
 }
 
+
 def collect(x, multi=False):
     """
     Collect single part geometries into their Multi* counterpart
@@ -32,12 +34,11 @@ def collect(x, multi=False):
     # Point and MultiPoint... or even just MultiPoint
     t = x[0].type
     if not all(g.type == t for g in x):
-        raise ValueError('Geometry type must be homogenous')
-    if len(x) > 1 and t.startswith('Multi'):
-        raise ValueError(
-            'Cannot collect {0}. Must have single geometries'.format(t))
+        raise ValueError("Geometry type must be homogeneous")
+    if len(x) > 1 and t.startswith("Multi"):
+        raise ValueError("Cannot collect {0}. Must have single geometries".format(t))
 
-    if len(x) == 1 and (t.startswith('Multi') or not multi):
+    if len(x) == 1 and (t.startswith("Multi") or not multi):
         # If there's only one single part geom and we're not forcing to
         # multi, then just return it
         return x[0]
